package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesPartPullService;
import cn.estsh.i3plus.ext.mes.api.base.IMesPullingOrderInfoService;
import cn.estsh.i3plus.ext.mes.apiservice.utils.MesException;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.base.util.StringUtil;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.repository.*;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.util.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

@Service
@Slf4j
public class MesPullingOrderInfoService extends BaseMesService<MesPullingOrderInfo> implements IMesPullingOrderInfoService {

    @Autowired
    private MesPullingOrderPartInfoRepository mesPullingOrderPartInfoRepository;
    @Autowired
    private MesPartPullRepository mesPartPullRDao;
    @Autowired
    private MesPartPullDetailRepository mesPartPullDetailRDao;
    @Autowired
    private MesWorkOrderRepository mesWorkOrderRepository;
    @Autowired
    private MesWorkCenterRepository mesWorkCenterRepository;
    protected void setPackQueryBean(MesPullingOrderInfo bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringEqualPack(bean.getPullCode(), "pullCode", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getPullingOrderNo(), "pullingOrderNo", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getWorkCenterCode(), "workCenterCode", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getWorkOrderNo(), "workOrderNo", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getPartNo(), "partNo", packBean);
    }

    @Override
    public void validateImport(List<MesPullingOrderInfo> beanList) {

        for (MesPullingOrderInfo item : beanList) {
            // 数据校验
            ValidatorBean.checkNotNull(item.getPullCode(), "拉动组代码不能为空");
            ValidatorBean.checkNotNull(item.getWorkCenterCode(), "产线代码不能为空");


            DdlPackBean seriesPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
            DdlPreparedPack.getStringEqualPack(item.getPullCode(), "pullCode", seriesPackBean);
            DdlPreparedPack.getStringEqualPack(item.getWorkOrderNo(), "workOrderNo", seriesPackBean);
            DdlPreparedPack.getStringEqualPack(item.getWorkCenterCode(), "workCenterCode", seriesPackBean);
            boolean flg = baseRDao.isExitByHql(seriesPackBean);
            if (flg) {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("工单【%s】拉动组代码【%s】已经存在，请检查数据", item.getWorkOrderNo(), item.getPullCode())
                        .build();
            }
        }
    }

    @Override
    public void doCreatePullOrderByPullGroupAndWorkOrder(String organizeCode, String userName) {
        //1.拉动组集合+拉动组明细集合pullingGroupList+ pullingpartlist
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        List<MesPartPull> pulliGroupList = mesPartPullRDao.findByHqlWhere(ddlPackBean);
        if (CollectionUtils.isEmpty(pulliGroupList)){
            return;
        }
        DdlPackBean ddlPackBeanDetail = DdlPackBean.getDdlPackBean(organizeCode);
        List<MesPartPullDetail> pullDetailList = mesPartPullDetailRDao.findByHqlWhere(ddlPackBeanDetail);
        if (CollectionUtils.isEmpty(pullDetailList)){
            return;
        }
        //2.客户物料集合 [PART_NO] in 拉动组明细物料集合
        //3.产线数据
        DdlPackBean workPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getInPackList(pulliGroupList.stream().map(MesPartPull::getWorkCenterCode).collect(Collectors.toList()), "workCenterCode", workPackBean);
        List<MesWorkCenter> mesWorkCenterList = mesWorkCenterRepository.findByHqlWhere(workPackBean);

    }

    @Override
    public Map doSPSForEquipment(String assemblyLine, String pullingGroupCode, String organizeCode) {
        Map resultMap = new HashMap();
        //查询拉动单状态为已扫描=20的 拉动单
        DdlPackBean seriesPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(pullingGroupCode, "pullCode", seriesPackBean);
        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.PULL_ORDER_STATUS.SPS_PULL.getValue(), "pullOrderStatus", seriesPackBean);
        DdlPreparedPack.getStringEqualPack(assemblyLine, "workCenterCode", seriesPackBean);
        seriesPackBean.setWhereAppend(seriesPackBean.getWhereAppend() + "  order by modifyDatetime desc");

        MesPullingOrderInfo mesPullingOrder  = baseRDao.getByProperty(seriesPackBean);
        if (StringUtil.isEmpty(mesPullingOrder)) {
            resultMap.put("success", false);
            String message = "产线" + assemblyLine + "拉动组" + pullingGroupCode + "数据为空，请检查数据";
            resultMap.put("message", message);
            return resultMap;
        }
        DdlPackBean partPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(pullingGroupCode, "pullCode", partPackBean);
        partPackBean.setWhereAppend(partPackBean.getWhereAppend() + "  order by modifyDatetime desc");

        //List<MesPullingOrderInfo> byHqlWhere = mesPullingOrderPartInfoRepository.findByHqlWhere(partPackBean);

        resultMap.put("",mesPullingOrder);
        resultMap.put("success", true);

        String message ="更新SPS状态成功，已发送SPS信息给设备! id:"+mesPullingOrder.getId()+  "Code:" + mesPullingOrder.getPullingOrderNo() + "产线" + assemblyLine + "拉动组" + pullingGroupCode + "";
        resultMap.put("message", message);
        //更新主表状态 源系统为25配料完成  现在-【已发送设备-30】
        mesPullingOrder.setPullOrderStatus(MesExtEnumUtil.PULL_ORDER_STATUS.SPS_PULL.getValue());
        ConvertBean.serviceModelUpdate(mesPullingOrder, "WS.SPS");
        baseRDao.update(mesPullingOrder);

        return resultMap;
    }
}
